#################################s#o#q######################################
## by Arthur Chan @ soqhome.com
#############################################################################

from AccessControl import ClassSecurityInfo
from Globals import InitializeClass, package_home
from OFS.SimpleItem import SimpleItem
from Products.CMFCore.CMFCorePermissions  import ManagePortal, ModifyPortalContent, View
from Products.CMFCore.utils import UniqueObject, getToolByName
from Products.PageTemplates.PageTemplateFile import PageTemplateFile

from Products.SoQClub.config import *

import calendar

from DateTime import DateTime


class SoQClubTool(UniqueObject, SimpleItem):
    """ CMF Syndication Client  """

    id        = 'soqclub_tool'
    meta_type = 'SoQClub Tool'
    security = ClassSecurityInfo()
    plone_tool = 1

    manage_options=(
        ({ 'label'   : 'Config',
           'action'  : 'manage_configForm',
           },
         ) + SimpleItem.manage_options
        )

    security.declareProtected( ManagePortal
                             , 'manage_configForm' )    
    
    manage_configForm = PageTemplateFile('www/config', globals(), __name__='manage_configForm')

    def __init__(self):
        self._reset()

    def _reset(self):
        ##### Portal Data #####
        self.SOQPORTAL_CODECS   = SOQPORTAL_CODECS
        
        ##### Channel Data #####
        self.SOQCHANNEL_ENABLE_SYNDICATION	= SOQCHANNEL_ENABLE_SYNDICATION
        self.SOQCHANNEL_CLUSTERMASTER   = SOQCHANNEL_CLUSTERMASTER
        
        self.SOQCHANNEL_SLOTS_BASICLEFT   = SOQCHANNEL_SLOTS_BASICLEFT
        self.SOQCHANNEL_SLOTS_BASICRIGHT   = SOQCHANNEL_SLOTS_BASICRIGHT
        ##### Activity Data #####
        self.SOQAUXILIARY_SLOTSPOSITION   = SOQAUXILIARY_SLOTSPOSITION
        self.SOQAUXILIARY_SLOTS_BASICLEFT   = SOQAUXILIARY_SLOTS_BASICLEFT
        self.SOQAUXILIARY_SLOTS_BASICRIGHT   = SOQAUXILIARY_SLOTS_BASICRIGHT


    def _set(self, REQUEST=None, **kwargs):
        if REQUEST==None:
            arglist= kwargs
        else:
            arglist = REQUEST
        
        ##### Portal Data #####
        self.SOQPORTAL_CODECS   	= arglist['SOQPORTAL_CODECS']
        
        ##### Channel Data #####
        self.SOQCHANNEL_ENABLE_SYNDICATION	= arglist['SOQCHANNEL_ENABLE_SYNDICATION']
        self.SOQCHANNEL_CLUSTERMASTER   = arglist['SOQCHANNEL_CLUSTERMASTER']
        
        self.SOQCHANNEL_SLOTS_BASICLEFT   = arglist['SOQCHANNEL_SLOTS_BASICLEFT']
        self.SOQCHANNEL_SLOTS_BASICRIGHT   = arglist['SOQCHANNEL_SLOTS_BASICRIGHT']
        
        ##### Activity Data #####
        self.SOQAUXILIARY_SLOTS_BASICLEFT   = arglist['SOQAUXILIARY_SLOTS_BASICLEFT']
        self.SOQAUXILIARY_SLOTS_BASICRIGHT   = arglist['SOQAUXILIARY_SLOTS_BASICRIGHT']
        
    security.declarePublic('getCodecs')
    def getCodecs(self):
        return self.SOQPORTAL_CODECS
    
    def manage_configClub(self, submit, \
                         REQUEST=None, **kwargs):
        """config this puppy"""
        if submit == "Set Config":
            self._set(REQUEST=REQUEST, **kwargs)
        elif submit == "Reset Config":
            self._reset()

        if REQUEST:
            return REQUEST.RESPONSE.redirect(self.absolute_url() + \
                                             "/manage_configForm?portal_status_message=Config+Updated")


    #############################################################################
    ##                             Calendar                                    ##
    #############################################################################

    security.declarePublic('soqGetEventsForCalendar')
    def soqGetEventsForCalendar(self, month='1', year='2002'):
        """ recreates a sequence of weeks, by days each day is a mapping.
            {'day': #, 'url': None}
        """
        year=int(year)
        month=int(month)
        # daysByWeek is a list of days inside a list of weeks, like so:
        # [[0, 1, 2, 3, 4, 5, 6],
        #  [7, 8, 9, 10, 11, 12, 13],
        #  [14, 15, 16, 17, 18, 19, 20],
        #  [21, 22, 23, 24, 25, 26, 27],
        #  [28, 29, 30, 31, 0, 0, 0]]
        daysByWeek=calendar.monthcalendar(year, month)
        weeks=[]
        
        events=self.soqcatalog_getevents(year, month)
    
        for week in daysByWeek:
            days=[]
            for day in week:
                if events.has_key(day):
                    days.append(events[day])
                else:
                    days.append({'day': day, 'event': 0, 'eventslist':[]})
                
            weeks.append(days)
            
        return weeks

    security.declarePublic('soqcatalog_getevents')
    def soqcatalog_getevents(self, year, month):
        """ I added the criteria that all catalog matched results should have the path within the channel/activity """
        
        """ given a year and month return a list of days that have events """
        first_date=DateTime(str(month)+'/1/'+str(year))
        last_day=calendar.monthrange(year, month)[1]
        ## This line was cropping the last day of the month out of the
        ## calendar when doing the query
	## last_date=DateTime(str(month)+'/'+str(last_day)+'/'+str(year))
	last_date=first_date + last_day    
        portal_calendar = self.portal_calendar
	query=self.portal_catalog(portal_type=portal_calendar.calendar_types,
                                  review_state='published',	                          
                                  start=(first_date, last_date),
                                  start_usage='range:min:max',
                                  sort_on='start')
        # I don't like doing two searches
        # What i want to do is
        # start date => 1/1/2002 and start date <= 31/1/2002
        # or
        # end date => 1/1/2002 and end date <= 31/1/2002
        # but I don't know how to do that in one search query :(  - AD

        # if you look at calendar_slot you can see how to do this in 1 query - runyaga
        query+=self.portal_catalog(portal_type=portal_calendar.calendar_types,
                                   review_state='published',
                                   end=(first_date, last_date),
                                   end_usage='range:min:max',
                                   sort_on='end')
        
        # compile a list of the days that have events
        eventDays={}
        for daynumber in range(1, 32): # 1 to 31
            eventDays[daynumber] = {'eventslist':[], 'event':0, 'day':daynumber}
        includedevents = []
        
        chanpath = self.getChannelPath()
        
        for result in query:
                
            if result.getRID() in includedevents:
                break
            else:
                ### added code for path criteria
                obj = result.getObject()
                path =  obj.absolute_url()
                
                if path.find(chanpath) < 0:
                    break
                
                includedevents.append(result.getRID())
            event={}
            # we need to deal with events that end next month
            if  result.end.month() != month:  # doesn't work for events that last ~12 months - fix it if it's a problem, otherwise ignore
                eventEndDay = last_day
                event['end'] = None
            else:
                eventEndDay = result.end.day()
                event['end'] = result.end.Time()
            # and events that started last month
            if result.start.month() != month:  # same as above re: 12 month thing
                eventStartDay = 1
                event['start'] = None
            else:
                eventStartDay = result.start.day()
                event['start'] = result.start.Time()

            event['title'] = result.Title or result.id
            if eventStartDay != eventEndDay:
                allEventDays = range(eventStartDay, eventEndDay+1)
                eventDays[eventStartDay]['eventslist'].append({'end':None, 'start':result.start.Time(), 'title':result.Title})
                eventDays[eventStartDay]['event'] = 1
                for eventday in allEventDays[1:-1]:
                    eventDays[eventday]['eventslist'].append({'end':None, 'start':None, 'title':result.Title})
                    eventDays[eventday]['event'] = 1
                eventDays[eventEndDay]['eventslist'].append({'end':result.end.Time(), 'start':None, 'title':result.Title})
                eventDays[eventEndDay]['event'] = 1
            else:
                eventDays[eventStartDay]['eventslist'].append(event)
                eventDays[eventStartDay]['event'] = 1
            # This list is not uniqued and isn't sorted
            # uniquing and sorting only wastes time
            # and in this example we don't need to because
            # later we are going to do an 'if 2 in eventDays'
            # so the order is not important.
            # example:  [23, 28, 29, 30, 31, 23]
        return eventDays


InitializeClass(SoQClubTool)
