#################################s#o#q######################################
## by Arthur Chan @ soqhome.com
#############################################################################


from Products.SoQClub.config import *
from Products.Archetypes.public import *
from Products.CMFCore import CMFCorePermissions
from OFS.Image import File
from AccessControl import ClassSecurityInfo

#############################################################################
##                             Setting                                     ##
#############################################################################

schema = BaseSchema +  Schema((
    StringField('title',
             required=1,
             searchable=1,
        ),
    TextField('body',
             searchable=1,
             required=1,
             default_content_type = 'text/html', 
             default_output_type = 'text/html', 
             widget=VisualWidget(label="Description",
                               description="Type in the content for the video",
                       ),
        ),
    StringField ('mimetype',
             vocabulary=SOQVIDEO_MIMETYPE,
             required=1,
             default='video/x-ms-wmv',
             widget=SelectionWidget(label="File Format"),
        ),
    FileField('video',
             required=1,
             mutator='setVideo',
             primary=1,
             mimetype='video/x-ms-wmv',
             widget=FileWidget(label="File"),
        ),
    IntegerField('width',
             default=200,
             widget=IntegerWidget(label="Player Width"),
        ),
    IntegerField('height',
             default=150,
             widget=IntegerWidget(label="Player Height"),
        ),
    ))
    
class SoQVideo(BaseContent):
    """This is a Image object
    """
    security = ClassSecurityInfo()    
    schema = schema
    archetype_name = "SoQ Video"
    global_allow = 0
    actions = ({
        'id': 'view',
        'name': 'View',
        'action': 'soqvideo_view',
        'permissions': (CMFCorePermissions.View,) 
        },
        )
        
    #############################################################################
    ##                             Basics                                      ##
    #############################################################################
    
    security.declareProtected( CMFCorePermissions.ModifyPortalContent, 'setVideo' )        
    def setVideo(self, value):
        schema = self.Schema()
        field = schema.get('video', None)
        mimetype= self.REQUEST['mimetype']
        field.set(instance=self, value=value)
        
        self._FileField_types['video'] = mimetype
        
    #############################################################################
    ##                             Utils                                       ##
    #############################################################################

    security.declareProtected( CMFCorePermissions.View, 'getVideoFile' )
    def getVideoFile(self):
        schema = self.Schema()
        field = schema.get('video', None)
        mimetype = self.getMimetype()
        ext = self.getContentExtension(mimetype)
        data = field.get(self)
        RESPONSE = self.REQUEST.RESPONSE
        RESPONSE.setHeader('Content-Type', mimetype)
        RESPONSE.setHeader('Content-Length', len(data))
        RESPONSE.setHeader('Content-Disposition', 'attachment; filename=%s.%s' % (self.getId(), ext))

        return data
        
    security.declarePublic( 'getContentExtension' )
    def getContentExtension(self, mimetype=''):
        ret = ''
        if mimetype =='':
            mimetype = self.getMimetype()
            
        if mimetype == 'application/vnd.rn-realmedia':
            ret = 'rm'
        elif mimetype =='video/x-ms-wmv':
            ret = 'wmv'
        elif mimetype =='video/quicktime':
            ret = 'mov'
        return ret
           
registerType(SoQVideo)
